/*
 * Written by Sean Kelly
 * Placed into Public Domain
 */

module std.c.posix.ucontext;

private import std.c.config;
public import std.c.posix.signal; // for sigset_t, stack_t

extern (C):

//
// XOpen (XSI)
//
/*
mcontext_t

struct ucontext_t
{
    ucontext_t* uc_link;
    sigset_t    uc_sigmask;
    stack_t     uc_stack;
    mcontext_t  uc_mcontext;
}
*/

version( linux )
{
    private
    {
        struct _libc_fpreg
        {
          ushort[4] significand;
          ushort    exponent;
        }

        struct _libc_fpstate
        {
          c_ulong           cw;
          c_ulong           sw;
          c_ulong           tag;
          c_ulong           ipoff;
          c_ulong           cssel;
          c_ulong           dataoff;
          c_ulong           datasel;
          _libc_fpreg[8]    _st;
          c_ulong           status;
        }

        const auto NGREG = 19;

        alias int               greg_t;
        alias greg_t[NGREG]     gregset_t;
        alias _libc_fpstate*    fpregset_t;
    }

    struct mcontext_t
    {
        gregset_t   gregs;
        fpregset_t  fpregs;
        c_ulong     oldmask;
        c_ulong     cr2;
    }

    struct ucontext_t
    {
        c_ulong         uc_flags;
        ucontext_t*     uc_link;
        stack_t         uc_stack; // note that this requires XSI extensions
        mcontext_t      uc_mcontext;
        sigset_t        uc_sigmask;
        _libc_fpstate   __fpregs_mem;
    }
}

//
// Obsolescent (OB)
//
/*
int  getcontext(ucontext_t*);
void makecontext(ucontext_t*, void function(), int, ...);
int  setcontext(ucontext_t*);
int  swapcontext(ucontext_t*, ucontext_t*);
*/